from django.db import models
from django.contrib.contenttypes.models import ContentType


class Publisher(models.Model):
    title = models.CharField(
        max_length=64
    )
    content_type = models.ForeignKey(
        ContentType,
        editable=False,
        null=True
    )
    class_name = models.CharField(
        max_length=32,
        editable=False,
        null=True
    )

    def __unicode__(self):
        return "%s - %s" % (self._meta.verbose_name, self.title)

    def as_leaf_class(self):
        """
        Returns the leaf class no matter where the calling instance is in the
        inheritance hierarchy.
        Inspired by http://www.djangosnippets.org/snippets/1031/
        """
        try:
            return self.__getattribute__(self.class_name.lower())
        except AttributeError:
            content_type = self.content_type
            model = content_type.model_class()
            if(model == ModelBase):
                return self
            return model.objects.get(id=self.id)

    def publish(self, instance):
        self.as_leaf_class().publish(instance)

    def save(self, *args, **kwargs):
        # set leaf class content type
        if not self.content_type:
            self.content_type = ContentType.objects.get_for_model(\
                    self.__class__)

        # set leaf class class name
        if not self.class_name:
            self.class_name = self.__class__.__name__

        super(Publisher, self).save(*args, **kwargs)


class Buzz(Publisher):
    # TODO: Add setup fields and methods to publish
    class Meta:
        ordering = ('title',)
        verbose_name = 'Buzz'
        verbose_name_plural = 'Buzz'


class Facebook(Publisher):
    # TODO: Add setup fields and methods to publish
    class Meta:
        ordering = ('title',)
        verbose_name = 'Facebook'
        verbose_name_plural = 'Facebook'


class Mobile(Publisher):
    # TODO: Add setup fields and methods to publish
    class Meta:
        ordering = ('title',)
        verbose_name = 'Mobile'
        verbose_name_plural = 'Mobile'


class Twitter(Publisher):
    # TODO: Add setup fields and methods to publish
    class Meta:
        ordering = ('title',)
        verbose_name = 'Twitter'
        verbose_name_plural = 'Twitter'


class SocialBookmark(Publisher):
    # TODO: Add setup fields and methods to publish
    pass


class Web(Publisher):
    # TODO: Add setup fields and methods to publish
    class Meta:
        ordering = ('title',)
        verbose_name = 'Web'
        verbose_name_plural = 'Web'

# TODO: Add publishers for Mxit, Grid, Jil,
# Geo (Foursquare), Vlive, MTNPlay, V360, MyWeb.



from django.contrib import admin

from publisher.models import Buzz, Facebook, Mobile, SocialBookmark, Twitter, \
        Web

admin.site.register(Buzz)
admin.site.register(Facebook)
admin.site.register(Mobile)
admin.site.register(SocialBookmark)
admin.site.register(Twitter)
admin.site.register(Web)



from django.db.models import signals

from publisher.models import Publisher


def publish(sender, instance, action, reverse, model, pk_set, **kwargs):
    if model == Publisher:
        if pk_set and action == 'post_add':
            publishers = Publisher.objects.filter(pk__in=pk_set)
            for publisher in publishers:
                publisher.publish(instance)

signals.m2m_changed.connect(publish)



from setuptools import setup, find_packages
from setuptools.command.test import test

def run_tests(self):
    from setuptest.runtests import runtests
    return runtests(self)
test.run_tests = run_tests

setup(
    name='django-publisher',
    version='0.0.2',
    description='Django external publishing app.',
    long_description = open('README.rst', 'r').read() + open('AUTHORS.rst', 'r').read() + open('CHANGELOG.rst', 'r').read(),
    author='Praekelt Foundation',
    author_email='dev@praekelt.com',
    license='BSD',
    packages = find_packages(),
    test_suite="publisher.tests",
    tests_require=[
        'django-setuptest',
    ],
    #dependency_links = [
    #    'http://github.com/sciyoshi/pyfacebook/tarball/master#egg=pyfacebook',
    #],
    #install_requires = [
    #    'pyfacebook',
    #],
    include_package_data=True,
    classifiers = [
        "Programming Language :: Python",
        "License :: OSI Approved :: BSD License",
        "Development Status :: 4 - Beta",
        "Operating System :: OS Independent",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
)
